@@ -7,7 +7,7 @@ tol_large = 1e-10
 test_that(".meta_gen correctly estimates the pooled effect size for d values with raw information", {
   df <- subset(df.SMD, factor == "Pharmacological", select = -c(value, se, ci_lo, ci_up))
 
-  umb <- umbrella(df, method.var = "REML")[[1]]$random
+  umb <- umbrella(df, method.var = "REML")[[1]]$ma_results
 
   df_mfr = metafor::escalc(m1i = mean_cases, m2i = mean_controls,
                            sd1i = sd_cases, sd2i = sd_controls,
@@ -26,7 +26,7 @@ test_that(".meta_gen correctly estimates the pooled effect size for d values val
 
   df <- subset(df.SMD, factor == "Pharmacological", select = -c(ci_lo, ci_up, mean_cases, mean_controls, sd_cases, sd_controls))
 
-  umb <- .quiet(umbrella(df, method.var = "REML")[[1]]$random)
+  umb <- .quiet(umbrella(df, method.var = "REML")[[1]]$ma_results)
 
   meta <- metafor::rma.uni(yi = .estimate_g_from_d(df$value, df$n_cases, df$n_controls)$value,
                            sei = .estimate_g_from_d(df$value, df$n_cases, df$n_controls)$se,
@@ -46,8 +46,117 @@ test_that(".meta_gen correctly estimates the pooled effect size for d values and
   meta <- meta::metagen(TE = meta_umb[[1]]$x$value, seTE = meta_umb[[1]]$x$se,
                         hakn = TRUE, method.tau = "DL")
 
-  expect_equal(meta_umb[[1]]$random$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
-  expect_equal(meta_umb[[1]]$random$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+})
+
+###########
+### SMC ###
+###########
+
+# SMC - standard analysis
+test_that(".meta_gen correctly estimates the pooled effect size for d values with change information", {
+  df <- subset(df.SMC, factor == "Physically_active", select = -c(value, se, ci_lo, ci_up, mean_pre_cases, mean_pre_controls))
+
+  umb <- umbrella(df, method.var = "REML")[[1]]$ma_results
+
+  df$r = 0
+  smc_cases <- metafor::escalc(m1i = df$mean_change_cases,
+                               sd1i = df$sd_change_cases,
+                               ni = df$n_cases,
+                               m2i = df$r,
+                               sd2i = df$r,
+                               ri = df$r,
+                               measure = "SMCC", vtype = "LS")
+  smc_controls <- metafor::escalc(m1i = df$mean_change_controls,
+                                  sd1i = df$sd_change_controls,
+                                  ni = df$n_controls,
+                                  m2i = df$r,
+                                  sd2i = df$r,
+                                  ri = df$r,
+                                  measure = "SMCC", vtype = "LS")
+
+  smcc = smc_cases$yi - smc_controls$yi
+  se = sqrt(smc_cases$vi + smc_controls$vi)
+
+  meta <- metafor::rma.uni(yi = smcc, sei = se, method = "REML")
+
+  expect_equal(as.numeric(as.character(umb$value)), as.numeric(as.character(meta$beta)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(umb$p.value)), as.numeric(as.character(meta$pval)), tolerance = tol_large)
+})
+
+test_that(".meta_gen correctly estimates the pooled effect size for d values with raw information", {
+  df <- subset(df.SMC, factor == "Physically_active", select = -c(value, se, ci_lo, ci_up, mean_change_cases, sd_change_cases, sd_change_controls))
+
+  umb <- suppressWarnings(umbrella(df, method.var = "REML", pre_post_cor = 0.5)[[1]]$ma_results)
+
+  df$pre_post_cor = 0.5
+  smc_cases <- metafor::escalc(m1i = df$mean_cases,
+                               sd1i = df$sd_cases,
+                               ni = df$n_cases,
+                               m2i = df$mean_pre_cases,
+                               sd2i = df$sd_pre_cases,
+                               ri = df$pre_post_cor,
+                               measure = "SMCC", vtype = "LS")
+  smc_controls <- metafor::escalc(m1i = df$mean_controls,
+                                  sd1i = df$sd_controls,
+                                  ni = df$n_controls,
+                                  m2i = df$mean_pre_controls,
+                                  sd2i = df$sd_pre_controls,
+                                  ri = df$pre_post_cor,
+                                  measure = "SMCC", vtype = "LS")
+
+  smcc = smc_cases$yi - smc_controls$yi
+  se = sqrt(smc_cases$vi + smc_controls$vi)
+
+  meta <- metafor::rma.uni(yi = smcc, sei = se, method = "REML")
+
+  expect_equal(as.numeric(as.character(umb$value)), as.numeric(as.character(meta$beta)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(umb$p.value)), as.numeric(as.character(meta$pval)), tolerance = tol_large)
+})
+
+# SMC - hksj estimator
+test_that(".meta_gen correctly estimates the pooled effect size for d values and hksj estimator", {
+
+  df <- subset(df.SMC, factor == "Physically_active")
+
+  meta_umb <- suppressWarnings(umbrella(df, method.var = "hksj"))
+  meta <- meta::metagen(TE = meta_umb[[1]]$x$value, seTE = meta_umb[[1]]$x$se,
+                        hakn = TRUE, method.tau = "DL")
+
+  expect_equal(meta_umb[[1]]$ma_results$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+})
+
+
+###########
+#### Z ####
+###########
+
+test_that(".meta_gen correctly estimates the pooled effect size for z values from r", {
+  df <- subset(df.R, factor == "gestational_diabetes")
+
+  umb <- suppressWarnings(umbrella(df, method.var = "REML")[[1]]$ma_results)
+
+  z_mfr <- metafor::escalc(ri = df$value,
+                               ni = df$n_sample,
+                               measure = "ZCOR")
+
+  meta <- metafor::rma.uni(yi = .as_numeric(z_mfr$yi), sei = sqrt(z_mfr$vi), method = "REML")
+
+  expect_equal(as.numeric(as.character(umb$value)), as.numeric(as.character(meta$beta)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(umb$p.value)), as.numeric(as.character(meta$pval)), tolerance = tol_large)
+})
+test_that(".meta_gen correctly estimates the pooled effect size for z values and hksj estimator", {
+
+  df <- subset(df.R, factor == "gestational_diabetes")
+
+  meta_umb <- suppressWarnings(umbrella(df, method.var = "hksj"))
+  meta <- meta::metagen(TE = meta_umb[[1]]$x$value, seTE = meta_umb[[1]]$x$se,
+                        hakn = TRUE, method.tau = "DL")
+
+  expect_equal(meta_umb[[1]]$ma_results$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
 })
 
 
@@ -60,7 +169,7 @@ test_that("meta_gen_log correctly estimates the pooled effect size from 2x2 tabl
 
   df <- subset(df.OR, factor == "ASD", select = -c(value, ci_lo, ci_up))
 
-  umb <- umbrella(df, method.var = "REML")[[1]]$random
+  umb <- umbrella(df, method.var = "REML")[[1]]$ma_results
   meta <- metafor::rma.uni(ai = n_cases_exp, bi = n_cases_nexp,
                            ci = n_controls_exp, di = n_controls_nexp,
                            data = df, method = "REML", measure = "OR")
@@ -79,8 +188,8 @@ test_that("meta_gen_log correctly estimates the pooled effect size from 2x2 tabl
   meta <- meta::metagen(TE = log(meta_umb[[1]]$x$value), seTE = meta_umb[[1]]$x$se,
                         hakn = TRUE, method.tau = "DL")
 
-  expect_equal(meta_umb[[1]]$random$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
-  expect_equal(meta_umb[[1]]$random$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
 })
 
 # OR - generic
@@ -93,7 +202,7 @@ test_that(".meta_gen_log correctly estimates the pooled effect size for OR value
                                                    n_controls_exp, n_controls_nexp))
   df$se <- se[which(df.OR$factor=="ASD")]
 
-  umb <- umbrella(df, method.var = "REML")[[1]]$random
+  umb <- umbrella(df, method.var = "REML")[[1]]$ma_results
 
   meta <- metafor::rma.uni(yi = log(value), sei = se,
                            data = df, method = "REML", measure = "OR")
@@ -111,7 +220,7 @@ test_that(".meta_gen_log correctly estimates the pooled effect size", {
 
   df <- subset(df.RR, select = -c(value, ci_lo, ci_up))
 
-  umb <- umbrella(df, method.var = "REML")[[1]]$random
+  umb <- umbrella(df, method.var = "REML")[[1]]$ma_results
 
   meta <- metafor::rma.uni(ai = n_cases_exp, n1i = n_exp,
                            ci = n_cases_nexp, n2i = n_nexp,
@@ -131,8 +240,8 @@ test_that(".meta_gen_log correctly estimates the pooled effect size with hksj es
   meta <- meta::metagen(TE = log(meta_umb[[1]]$x$value), seTE = meta_umb[[1]]$x$se,
                         hakn = TRUE, method.tau = "DL")
 
-  expect_equal(meta_umb[[1]]$random$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
-  expect_equal(meta_umb[[1]]$random$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
 })
 
 # RR - generic for multilevel
@@ -148,7 +257,7 @@ test_that(".meta_gen_log correctly estimates the pooled effect size for RR value
                                   n_exp, n_nexp))
   df$se <- se
 
-  umb <- .quiet(umbrella(df, method.var = "REML")[[1]]$random)
+  umb <- .quiet(umbrella(df, method.var = "REML")[[1]]$ma_results)
 
   meta <- metafor::rma.uni(yi = log(value), sei = se,
                            data = df, method = "REML", measure = "RR")
@@ -172,8 +281,8 @@ test_that(".meta_gen_log correctly estimates the pooled effect size", {
                            x2i = n_cases_nexp, t2i = time_nexp,
                            data = df, method = "REML", measure = "IRR")
 
-  expect_equal(as.numeric(as.character(umb[[1]]$random$value)), as.numeric(as.character(meta$beta)), tolerance = tol_large)
-  expect_equal(as.numeric(as.character(umb[[1]]$random$p.value)), as.numeric(as.character(meta$pval)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(umb[[1]]$ma_results$value)), as.numeric(as.character(meta$beta)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(umb[[1]]$ma_results$p.value)), as.numeric(as.character(meta$pval)), tolerance = tol_large)
 })
 
 
@@ -185,8 +294,8 @@ test_that(".meta_irr correctly estimates the pooled effect size with hksj estima
   meta <- meta::metagen(TE = log(meta_umb[[1]]$x$value), seTE = meta_umb[[1]]$x$se,
                         sm = "IRR", hakn = TRUE, method.tau = "DL")
 
-  expect_equal(meta_umb[[1]]$random$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
-  expect_equal(meta_umb[[1]]$random$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$value, as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(meta_umb[[1]]$ma_results$p.value, as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
 })
 
 # IRR - standard analysis
@@ -194,7 +303,7 @@ test_that(".meta_irr correctly estimates the pooled effect size", {
 
   df <- subset(df.IRR, select = -c(value, ci_lo, ci_up))
 
-  umb <- umbrella(df, method.var = "REML")[[1]]$random
+  umb <- umbrella(df, method.var = "REML")[[1]]$ma_results
 
   meta <- metafor::rma.uni(x1i = n_cases_exp, t1i = time_exp,
                            x2i = n_cases_nexp, t2i = time_nexp,
@@ -214,8 +323,8 @@ test_that(".meta_irr correctly estimates the pooled effect size with hksj estima
   meta <- meta::metagen(TE = log(meta_umb[[1]]$x$value), seTE = meta_umb[[1]]$x$se,
                         sm = "IRR", hakn = TRUE, method.tau = "DL")
 
-  expect_equal(as.numeric(as.character(meta_umb[[1]]$random$value)), as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
-  expect_equal(as.numeric(as.character(meta_umb[[1]]$random$p.value)), as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(meta_umb[[1]]$ma_results$value)), as.numeric(as.character(meta$TE.random)), tolerance = tol_large)
+  expect_equal(as.numeric(as.character(meta_umb[[1]]$ma_results$p.value)), as.numeric(as.character(meta$pval.random)), tolerance = tol_large)
 })
 
 # IRR - generic for multilevel
@@ -227,7 +336,7 @@ test_that(".meta_gen_log correctly estimates the pooled effect size for IRR valu
                                   time_exp, time_nexp))
   df$se <- se
 
-  umb <- .quiet(umbrella(df, method.var = "REML")[[1]]$random)
+  umb <- .quiet(umbrella(df, method.var = "REML")[[1]]$ma_results)
 
   meta <- metafor::rma.uni(yi = log(value), sei = se,
                            data = df, method = "REML")
@@ -245,7 +354,7 @@ test_that(".meta_gen_log correctly estimates the pooled effect size for HR value
   df <- subset(df.HR, factor == "Yoga")
   df$se <- se[which(df.HR$factor == "Yoga")]
 
-  umb <- umbrella(df, method.var = "REML", verbose = FALSE)[[1]]$random
+  umb <- umbrella(df, method.var = "REML", verbose = FALSE)[[1]]$ma_results
 
   meta <- metafor::rma.uni(yi = log(value), sei = se,
                            data = df, method = "REML")
